#include "lowlevel.h"
#include "highlevel.h"
#include "font5x7s.h"

/* High level functions */

void LCD_home ( void )
{
	LCD_SelSide(1);
	LCD_WriteCmd (0x40);			// address 0
	LCD_WriteCmd (0xC0);			// display starts at address 0
	LCD_SelSide(0);
	LCD_WriteCmd (0x40);			// address 0
	LCD_WriteCmd (0xC0);			// display starts at address 0
}

void LCD_fill(unsigned char pattern)
{
unsigned char i,page;
	LCD_home();										// address 0,0
	for(page=0;page<8;page++) {
		LCD_SelSide(0);							// select left
		LCD_WriteCmd (0xB8+page);				// set page address	
		for(i=0;i<64;i++)
			LCD_WriteData(pattern);				// fill with pattern
		LCD_SelSide(1);							// select right
		LCD_WriteCmd (0xB8+page);				// set page address	
		for(i=0;i<64;i++)
			LCD_WriteData(pattern);				// fill with pattern
	}
}

#define LCD_clear() LCD_fill(0)


void LCD_plot (unsigned char x, unsigned char y)
{
unsigned char page,row,dat;
	LCD_SelSide(x>63);				// 0-63 => side 0; 63>127 => side 1
	x=(x>63)?(x-64):x;				// correct x address if >63
	LCD_WriteCmd (0x40+x);				// address = x or x-64
	LCD_CalcPage(y,&page,&row);			// calculate page and row within page
	LCD_WriteCmd (0xB8+page);			// set page address
	LCD_ReadData();					// dummy read after address setting
	dat=LCD_ReadData();				// get data at that address
	dat|=(1<<row);					// set requested bit
	LCD_WriteCmd (0x40+x);				// set address again (reading increments counter)
	LCD_WriteData(dat);
}

void LCD_dump (unsigned char * __code imgdata)
{
unsigned char i,page;

	LCD_home();										// address 0,0
	for(page=0;page<8;page++) {
		LCD_SelSide(0);							// select left
		LCD_WriteCmd (0xB8+page);				// set page address	
		for(i=0;i<64;i++)
			LCD_WriteData(*(imgdata++));		// send left half for each page
		LCD_SelSide(1);							// select right
		LCD_WriteCmd (0xB8+page);				// set page address	
		for(i=0;i<64;i++)
			LCD_WriteData(*(imgdata++));		// send right half for each page
	}
}

void LCD_putchr ( char chr )
{
unsigned char i;
	for(i=0;i<5;i++)
		LCD_WriteData(font5x7s[5*(chr-0x20)+i]);	// send character
	LCD_WriteData(0);										// send intercharacter separator	
}

void LCD_printat (unsigned char row, unsigned char col, char *ptr)
{
unsigned char x;
	do {
		x=(col>9)?6*(col-10):6*col;				// correct x address if col>9
		if(col>9)										// 0-9 => side 0; 10>20 => side 1
			LCD_SelSide(1);
		else {
			LCD_SelSide(0);
			x+=4;											// 4 pixels offset at both sides
		}
		LCD_WriteCmd (0xB8+row);					// set page address
		LCD_WriteCmd (0x40+x);						// address = 6*col or 6*col-10 (4 pixels offset)
		LCD_putchr (*ptr++);
		col++;
	} 	while (*ptr);
}
